import { EventEmitter } from "events";
import type { Stream } from "stream";
export interface ProxyTargetDetailed {
    host: string;
    port: number;
    protocol?: string;
    hostname?: string;
    socketPath?: string;
    key?: string;
    passphrase?: string;
    pfx?: Buffer | string;
    cert?: string;
    ca?: string;
    ciphers?: string;
    secureProtocol?: string;
}
export type ProxyType = "ws" | "web";
export type ProxyTarget = ProxyTargetUrl | ProxyTargetDetailed;
export type ProxyTargetUrl = URL | string | {
    port: number;
    host: string;
};
export interface ServerOptions {
    target?: ProxyTarget;
    forward?: ProxyTargetUrl;
    agent?: any;
    ssl?: any;
    ws?: boolean;
    xfwd?: boolean;
    secure?: boolean;
    toProxy?: boolean;
    prependPath?: boolean;
    ignorePath?: boolean;
    localAddress?: string;
    changeOrigin?: boolean;
    preserveHeaderKeyCase?: boolean;
    auth?: string;
    hostRewrite?: string;
    autoRewrite?: boolean;
    protocolRewrite?: string;
    cookieDomainRewrite?: false | string | {
        [oldDomain: string]: string;
    };
    cookiePathRewrite?: false | string | {
        [oldPath: string]: string;
    };
    headers?: {
        [header: string]: string | string[] | undefined;
    };
    proxyTimeout?: number;
    timeout?: number;
    followRedirects?: boolean;
    selfHandleResponse?: boolean;
    buffer?: Stream;
}
export declare class ProxyServer extends EventEmitter {
    readonly ws: any;
    readonly web: any;
    private options;
    private webPasses;
    private wsPasses;
    private _server?;
    constructor(options?: ServerOptions);
    createRightProxy: (type: ProxyType) => Function;
    onError: (err: Error) => void;
    listen: (port: number, hostname?: string) => this;
    address: () => any;
    close: (cb?: Function) => void;
    before: (type: ProxyType, passName: string, cb: Function) => void;
    after: (type: ProxyType, passName: string, cb: Function) => void;
}
