"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProxyServer = void 0;
const http = __importStar(require("http"));
const https = __importStar(require("https"));
const web_incoming_1 = require("./passes/web-incoming");
const ws_incoming_1 = require("./passes/ws-incoming");
const events_1 = require("events");
const debug_1 = __importDefault(require("debug"));
const common_1 = require("./common");
const log = (0, debug_1.default)("http-proxy-3");
class ProxyServer extends events_1.EventEmitter {
    constructor(options = {}) {
        super();
        // createRightProxy - Returns a function that when called creates the loader for
        // either `ws` or `web`'s passes.
        this.createRightProxy = (type) => {
            log("createRightProxy", { type });
            return (options) => {
                return (...args /* req, res, [head], [opts] */) => {
                    const req = args[0];
                    log("proxy: ", { type, path: req.url });
                    const res = args[1];
                    const passes = type === "ws" ? this.wsPasses : this.webPasses;
                    let counter = args.length - 1;
                    let head;
                    let cb;
                    // optional args parse begin
                    if (typeof args[counter] === "function") {
                        cb = args[counter];
                        counter--;
                    }
                    let requestOptions;
                    if (!(args[counter] instanceof Buffer) && args[counter] !== res) {
                        // Copy global options, and overwrite with request options
                        requestOptions = { ...options, ...args[counter] };
                        counter--;
                    }
                    else {
                        requestOptions = { ...options };
                    }
                    if (args[counter] instanceof Buffer) {
                        head = args[counter];
                    }
                    for (const e of ["target", "forward"]) {
                        if (typeof requestOptions[e] === "string") {
                            requestOptions[e] = (0, common_1.toURL)(requestOptions[e]);
                        }
                    }
                    if (!requestOptions.target && !requestOptions.forward) {
                        this.emit("error", new Error("Must set target or forward"));
                        return;
                    }
                    for (const pass of passes) {
                        /**
                         * Call of passes functions
                         *     pass(req, res, options, head)
                         *
                         * In WebSockets case, the `res` variable
                         * refer to the connection socket
                         *    pass(req, socket, options, head)
                         */
                        if (pass(req, res, requestOptions, head, this, cb)) {
                            // passes can return a truthy value to halt the loop
                            break;
                        }
                    }
                };
            };
        };
        this.onError = (err) => {
            // Force people to handle their own errors
            if (this.listeners("error").length === 1) {
                throw err;
            }
        };
        this.listen = (port, hostname) => {
            log("listen", { port, hostname });
            this._server = this.options.ssl
                ? https.createServer(this.options.ssl, this.web)
                : http.createServer(this.web);
            if (this.options.ws) {
                this._server.on("upgrade", (req, socket, head) => {
                    this.ws(req, socket, head);
                });
            }
            this._server.listen(port, hostname);
            return this;
        };
        // if the proxy started its own http server, this is the address of that server.
        this.address = () => {
            return this._server?.address();
        };
        this.close = (cb) => {
            if (this._server == null) {
                cb?.();
                return;
            }
            // Wrap cb anb nullify server after all open connections are closed.
            this._server.close((err) => {
                this._server = null;
                cb?.(err);
            });
        };
        this.before = (type, passName, cb) => {
            if (type !== "ws" && type !== "web") {
                throw new Error("type must be `web` or `ws`");
            }
            const passes = type === "ws" ? this.wsPasses : this.webPasses;
            let i = false;
            passes.forEach((v, idx) => {
                if (v.name === passName) {
                    i = idx;
                }
            });
            if (i === false) {
                throw new Error("No such pass");
            }
            passes.splice(i, 0, cb);
        };
        this.after = (type, passName, cb) => {
            if (type !== "ws" && type !== "web") {
                throw new Error("type must be `web` or `ws`");
            }
            const passes = type === "ws" ? this.wsPasses : this.webPasses;
            let i = false;
            passes.forEach((v, idx) => {
                if (v.name === passName) {
                    i = idx;
                }
            });
            if (i === false) {
                throw new Error("No such pass");
            }
            passes.splice(i++, 0, cb);
        };
        log("creating a ProxyServer", options);
        options.prependPath = options.prependPath === false ? false : true;
        this.options = options;
        this.web = this.createRightProxy("web")(options);
        this.ws = this.createRightProxy("ws")(options);
        this.webPasses = Object.values(web_incoming_1.WEB_PASSES);
        this.wsPasses = Object.values(ws_incoming_1.WS_PASSES);
        this.on("error", this.onError);
    }
}
exports.ProxyServer = ProxyServer;
